@@ -26,15 +26,18 @@ if venv:
     repo_dir = os.path.dirname(venv)
     src_dir = os.path.join(repo_dir, os.path.split(repo_dir)[-1])
     sys.path.insert(0, src_dir)
+    logging.warning("within virtualenv")
     import config
 elif sys.argv[0] == 'virtualenv/bin/aegis':
     # Running by calling the virtualenv binary directly
     repo_dir = os.getcwd()
     src_dir = os.path.join(repo_dir, os.path.split(repo_dir)[-1])
     sys.path.insert(0, src_dir)
+    logging.warning("aegis cmdline")
     import config
 elif sys.argv[0] == '/usr/local/bin/aegis':
     repo_dir = os.getcwd()
+    logging.warning("/usr/local/bin")
     if os.path.exists(os.path.join(repo_dir, '.git')):
         src_dir = os.path.join(repo_dir, os.path.split(repo_dir)[-1])
         sys.path.insert(0, src_dir)
@@ -44,6 +47,18 @@ elif sys.argv[0] == '/usr/local/bin/aegis':
         sys.exit(1)
 
 
+### Note to self: aegis create will work better if the core web is web.py so we don't clobber snowballin.py
+# Needs templates dir, etc
+# Prompt y/n to clobber with diff
+#
+
+
+# Also need aegis install, to make system admin faster
+# Should all start with the /aegis stuff
+# Also the sql should all be there for hydra, reports, etc
+
+
+
 # Create a new spinoff of aegis
 def create(parser):
     args = parser.parse_args()
@@ -61,6 +76,10 @@ def create(parser):
     #if os.path.exists(create_dir):
     #    logging.error("AEGIS     Sorry that directory exists already. Exiting.")
     #    sys.exit(1)
+
+    # If the directory exists prompt the user
+    # You can run aegis create again to produce a new create in your repo. Then you can look at git diff to resolve and differences.
+
     if not os.path.exists(create_dir):
         os.mkdir(create_dir)
     create_etc_dir = os.path.join(create_dir, 'etc')
@@ -71,6 +90,7 @@ def create(parser):
     for entry in os.walk(tmpl_dir):
         basedir, subdirs, files = entry
         rebasedir = create_dir + basedir[basedir.find('aegis/tmpl')+10:]
+        logging.warning(rebasedir)
         if rebasedir.endswith('/aegis'):
             rebasedir = rebasedir[:-6] + '/' + app_name
         if not os.path.exists(rebasedir):
@@ -92,6 +112,8 @@ def create(parser):
                 if filename == 'aegis_prod.conf':
                     rebase_filename = app_name + '_prod.conf'
                 rebasepath = os.path.join(rebasedir, rebase_filename)
+                # XXX TODO diff output between one on filesystem
+                # XXX maybe prompt y/n to overwrite
                 with open(rebasepath, 'w') as writefd:
                     writefd.write(output)
     print ("GREAT SUCCESS!!")
@@ -232,110 +254,15 @@ def deploy(parser):
     # Make it so
     logging.info("Running aegis deploy   Version: %s   Env: %s", version, env)
     build = aegis.build.Build()
-    deploy_msg = None
-    while not deploy_msg:
-        deploy_msg = input(aegis.stdlib.cstr('Type in release notes for the deploy notification:\n', 'white'))
-    build.deploy(version, env=env, deploy_msg=deploy_msg)
-
-    # This could produce a diff and then ask
-    # Could also record the diff (!!)
-
-    # This is a separate deploy program here, which we can reference at least in concept for producing diffs
-
-    # Create temp_dir if not exists
-    #if not os.path.exists(temp_dir):
-    #    os.makedirs(temp_dir)
-    ## Run Deploy
-    #files, output = rsync(silent=True)
-    #display(files, output)
-    ## Command line options and directory layout in global scope
-    #define('diff', default=True, help='Show Diffs', type=bool)
-    #define('dry_run', default=True, help='Dry Run - make no changes', type=bool)
-    #config.initialize()
-    #src_dir = os.path.dirname(aegis.stdlib.absdir(__file__))
-    #exclude_file = os.path.join(src_dir, 'etc', 'deploy.excludes')
-    #environments = {'prod': '/srv/www/_prod/' % app_name,
-    #                'dev': '/srv/www/<appname>_dev/' % app_name}
-    #if options.env not in environments:
-    #    logging.error("Environment %s isn't on the deploy list.", options.env)
-    #    sys.exit()
-    #dest_dir = environments[options.env]
-    #temp_dir = '/tmp/%s/deploy' % os.getenv('USER')
-    #logging.info("Running deploy.py   Env: %s   Src: %s   Dest: %s   Dry Run: %s", options.env, src_dir, dest_dir, options.dry_run)
-    ## set up rsync and file list filtering
-    #def rsync(silent=False):
-    #    rsync_opts = '-vzrlDc'
-    #    if options.dry_run:
-    #        rsync_opts += 'n'
-    #    rsync_opts += ' -T %s --delay-updates --exclude-from=%s --delete' % (temp_dir, exclude_file)
-    #    cmd = "rsync %s %s/ %s" % (rsync_opts, src_dir, dest_dir)
-    #    if not silent:
-    #        aegis.stdlib.logw(rsync_opts, 'RSYNC_OPTS:')
-    #        aegis.stdlib.logw(src_dir, 'SRC:')
-    #        aegis.stdlib.logw(dest_dir, 'DEST:')
-    #        aegis.stdlib.logw(cmd, 'CMD:')
-    #    pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
-    #    output, error_code = pipe.communicate()
-    #    # filter it down to a simple file list
-    #    files = []
-    #    output = output.decode('utf-8')
-    #    for line in output.splitlines():
-    #        fn = filter_line(line)
-    #        if fn:
-    #            files.append(fn)
-    #    return files, output
-    #regexes = (
-    #    re.compile(r'^building file list ...'),
-    #    re.compile(r'^sent [\d,]+ bytes'),
-    #    re.compile(r'^total size is \d+'),
-    #    )
-    #def filter_line(line):
-    #    if line in ('', './', 'sending incremental file list'):
-    #        return ''
-    #    for regex in regexes:
-    #        if regex.search(line):
-    #            return ''
-    #    return line
-    #def diff_files(src_file, dest_file, colorize=True):
-    #    cmd = "diff -ub %s %s" % (dest_file, src_file)
-    #    pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
-    #    output, error_code = pipe.communicate()
-    #    output = output.decode('utf-8')
-    #    cdiff = []
-    #    for line in output.splitlines():
-    #        line = line.strip()
-    #        if not line: continue
-    #        if line.startswith('-'):
-    #            cdiff.append(aegis.stdlib.cstr(line, color='red'))
-    #        elif line.startswith('+') or line.startswith('*'):
-    #            cdiff.append(aegis.stdlib.cstr(line, color='green'))
-    #        else:
-    #            cdiff.append(line)
-    #    cdiff = '\n'.join(cdiff)
-    #    return cdiff
-    #def display(files, output):
-    #    for fn in files:
-    #        #if fn.endswith('/') or ' -> ' in fn:
-    #        if fn.endswith('/'):
-    #            continue
-    #        if fn.startswith('cannot delete'):
-    #            logging.warning(aegis.stdlib.cstr('- %s' % fn, color='magenta'))
-    #            continue
-    #        if fn.startswith('deleting '):
-    #            logging.warning(aegis.stdlib.cstr('- %s' % fn, color='magenta'))
-    #            continue
-    #        # Display useful diff information since this is deploying
-    #        src_file = (src_dir + '/' + fn).strip()   # '/' because of rsync slash oddity
-    #        dest_file = (dest_dir + fn).strip()
-    #        fn_status = fn
-    #        if not os.path.exists(dest_file):
-    #            fn_status += '   [NEW FILE]'
-    #        logging.warning(aegis.stdlib.cstr('+ writing %s ' % fn_status, color='cyan'))
-    #        if os.path.exists(dest_file):
-    #            if options.diff:
-    #                cdiff = diff_files(src_file, dest_file)
-    #                if cdiff:
-    #                    logging.warning(cdiff)
+    message = None
+    while not message:
+        message = input(aegis.stdlib.cstr('Type in release notes for the deploy notification:\n', 'white'))
+    # Save the user message and start the deploy/revert
+    build_row = aegis.model.Build.get_version(version)
+    build_row.set_message(message, 'deploy')
+    build_row = aegis.model.Build.get_version(version)
+    aegis.build.Build.start_deploy(build_row, os.getenv('SUDO_USER'))
+    build.deploy(version, env=env)
 
 
 def revert(parser):
@@ -357,7 +284,17 @@ def revert(parser):
     # Make it so
     logging.info("Running aegis revert   Env: %s", env)
     build = aegis.build.Build()
-    build.revert(env=env)
+    message = None
+    while not message:
+        message = input(aegis.stdlib.cstr('Type in release notes for the deploy notification:\n', 'white'))
+    # Save the user message and start the deploy/revert
+    build_row = aegis.model.Build.get_live_build(env)
+    build_row.set_message(message, 'revert')
+    build_row = aegis.model.Build.get_id(build_row['build_id'])
+    aegis.build.Build.start_revert(build_row, os.getenv('SUDO_USER'))
+    build_row = aegis.model.Build.get_id(build_row['build_id'])
+    build_row.set_output('revert', '')
+    build.revert(build_row)
 
 
 def initialize():
@@ -372,8 +309,7 @@ def initialize():
         config.initialize(args=sys.argv[1:])
     except Exception as ex:
         # No config, such as during aegis create shell command
-        logging.exception(ex)
-        remaining = tornado.options.parse_command_line()
+        remaining = tornado.options.parse_command_line(sys.argv[1:])
 
 
 def main():
