@@ -194,7 +194,9 @@ class PostgresConnection(object):
 
     def _execute(self, cursor, query, parameters, **kwargs):
         try:
+            aegis.stdlib.incr_start(aegis.stdlib.get_timer(), 'database')
             cursor.execute(query, parameters)
+            aegis.stdlib.incr_stop(aegis.stdlib.get_timer(), 'database')
             return
         except PgsqlUniqueViolation as ex:
             # UniqueViolation doesn't need to close connection, it needs to be handled in application
@@ -485,7 +487,10 @@ class MysqlConnection(object):
 
     def _execute(self, cursor, query, parameters):
         try:
-            return cursor.execute(query, parameters)
+            aegis.stdlib.incr_start(aegis.stdlib.get_timer(), 'database')
+            result = cursor.execute(query, parameters)
+            aegis.stdlib.incr_stop(aegis.stdlib.get_timer(), 'database')
+            return result
         except MysqlOperationalError as ex:
             logging.error("Error with MySQL on %s", self.host)
             logging.exception(ex)
