@@ -111,7 +111,8 @@ class HydraThread(threading.Thread):
     @staticmethod
     def signal_reset(signal, frm):
         logging.warning("SIGHUP received... clearing stale claims")
-        aegis.model.HydraQueue.clear_claims()
+        aegis.model.HydraType.clear_claims(minutes=self.stuck_minutes)
+        aegis.model.HydraQueue.clear_claims(minutes=self.stuck_minutes)
 
 
 class HydraHead(HydraThread):
@@ -121,6 +122,7 @@ class HydraHead(HydraThread):
         self.thread_name = 'HydraHead-%02d' % self.hydra_head_id
         self.hostname = socket.gethostname()
         HydraThread.__init__(self, name=self.thread_name)
+        self.hydra_type_maxlen = max([len(ht['hydra_type_name']) for ht in aegis.model.HydraType.scan()])
 
 
     def process(self):
@@ -140,6 +142,7 @@ class HydraHead(HydraThread):
                         hydra_type = aegis.model.HydraType.get_id(hydra_queue['hydra_type_id'])
                         #self.logw(claimed, "CLAIMED HYDRA QUEUE: %s  TYPE: %s  HOST: %s" % (hydra_queue['hydra_queue_id'], hydra_type['hydra_type_name'], hydra_queue['work_host']))
                         if not claimed: continue
+                        start_t = time.time()
                         # Hydra Magic: Find the hydra_type specific function in a subclass of HydraHead
                         if not hydra_type:
                             logging.error("Missing hydra type for hydra_type_id: %s", hydra_type)
@@ -147,6 +150,8 @@ class HydraHead(HydraThread):
                         if not hasattr(self, hydra_type['hydra_type_name']):
                             logging.error("Missing hydra function for hydra type: %s", hydra_type)
                             continue
+                        self.timer_obj = aegis.stdlib.TimerObj()
+                        aegis.stdlib.timer_start(self.timer_obj, 'hydra_queue_run')
                         self.iter_cnt += 1
                         work_fn = getattr(self, hydra_type['hydra_type_name'])
                         # Allow queue items to specify that they should run in a specific host and environment.
@@ -158,7 +163,8 @@ class HydraHead(HydraThread):
                         hydra_queue.incr_try_cnt()
                         hydra_queue.start()
                         result, work_cnt = work_fn(hydra_queue, hydra_type)
-                        #self.logw(work_cnt, "WORK CNT")
+                        end_t = time.time()
+                        exec_t_ms = (end_t - start_t) * 1000
                         if result:
                             hydra_queue.complete()
                         else:
@@ -166,8 +172,9 @@ class HydraHead(HydraThread):
                             hydra_queue.incr_error_cnt(minutes=15)
                             hydra_queue.unclaim()
                             continue
+                        aegis.stdlib.timer_stop(self.timer_obj, 'hydra_queue_run')
                         # Worker accounting
-                        logging.warning(self.log_line(hydra_type, work_cnt, " DONE"))
+                        logging.warning(self.log_line(hydra_type, work_cnt, self.timer_msg()))
                         self.processed_cnt += work_cnt
                     except Exception as ex:
                         logging.error("Exception when working on hydra_queue_id: %s", hydra_queue['hydra_queue_id'])
@@ -183,9 +190,22 @@ class HydraHead(HydraThread):
         finally:
             self.finish()
 
+    def timer_msg(self):
+        exec_t_ms = self.timer_obj._timers.get('_hydra_queue_run_exec_s') * 1000
+        net_t_ms = self.timer_obj._timers.get('_network_exec_s', 0) * 1000
+        db_t_ms = self.timer_obj._timers.get('_database_exec_s') * 1000
+        cpu_t_ms = exec_t_ms - net_t_ms - db_t_ms
+        exec_t_str = "%.3fms" % exec_t_ms
+        net_t_str = "%.3fms" % net_t_ms
+        db_t_str = "%.3fms" % db_t_ms
+        cpu_t_str = "%.3fms" % cpu_t_ms
+        msg = " DONE %12s  |  %12s cpu  %12s db  %12s net" % (exec_t_str, cpu_t_str, db_t_str, net_t_str)
+        return msg
 
     def log_line(self, hydra_type, work_cnt=0, msg=''):
-        return '%s %s %s %s' % (self.name, hydra_type['hydra_type_name'], ("%d" % work_cnt).rjust(8), msg)
+        line = '%s %s %s %s' % (self.name, format(hydra_type['hydra_type_name'], str(self.hydra_type_maxlen)), ("%d" % work_cnt).rjust(8), msg)
+        #self.logw(line, "LINE")
+        return line
 
 
     def housekeeping(self, hydra_queue, hydra_type):
@@ -193,7 +213,6 @@ class HydraHead(HydraThread):
         # enqueue clean_build for each deploy host
         hydra_type = aegis.model.HydraType.get_name('clean_build')
         for deploy_host in options.deploy_hosts:
-            self.logw(deploy_host, "DH")
             hydra_queue = {'hydra_type_id': hydra_type['hydra_type_id'], 'priority_ndx': hydra_type['priority_ndx'], 'work_dttm': aegis.database.Literal("NOW()"),
                            'work_host': deploy_host, 'work_env': aegis.config.get('env')}
             hydra_queue_id = aegis.model.HydraQueue.insert_columns(**hydra_queue)
@@ -246,6 +265,11 @@ class HydraHead(HydraThread):
 
 
     def clean_build(self, hydra_queue, hydra_type):
+        # Singleton check - if someone else claimed a different hydra_queue of the same hydra_type, hydra_queue needs to unclaim and stop
+        singleton = hydra_queue.singleton()
+        if singleton:
+            logging.error("clean_build already running")
+            return True, 0
         # run on every host to clean out builds that are leftover and taking up disk space
         logging.warning("RUNNING clean_build on %s env %s", hydra_queue['work_host'], hydra_queue['work_env'])
         build = aegis.build.Build()
@@ -276,7 +300,7 @@ class Hydra(HydraThread):
         HydraThread.__init__(self, name=self.thread_name)
         self.num_heads = 3
         self.hydra_head_cls = HydraHead
-
+        self.stuck_minutes = aegis.config.get('hydra_stuck_minutes') or 15
 
 
     def spawn_heads(self):
@@ -291,8 +315,8 @@ class Hydra(HydraThread):
         # When starting up, hydra_id 0 clears claims before spawning heads.
         if self.hydra_id == 0:
             logging.warning("%s clearing stale claims" % self.name)
-            aegis.model.HydraType.clear_claims()
-            aegis.model.HydraQueue.clear_claims()
+            aegis.model.HydraType.clear_claims(minutes=self.stuck_minutes)
+            aegis.model.HydraQueue.clear_claims(minutes=self.stuck_minutes)
             # If the hydra_type_id for this queue item has next_run_sql then it should be a singleton across the hydras.
             # This means set hydra_type['status'] = 'running' and set it back to 'live' after completion.
             logging.warning("%s clearing running jobs over 45 minutes old" % self.name)
@@ -326,13 +350,19 @@ class Hydra(HydraThread):
                             #if purged_completed:
                             #    logging.warning("%s queue purge deleted %s hydra_queue" % (self.thread_name, purged_completed))
                             # Log if there are expired queue items in the past...
-                            past_items = aegis.model.HydraQueue.past_items()
+                            past_items = aegis.model.HydraQueue.past_items(minutes=self.stuck_minutes)
                             if past_items and len(past_items):
-                                logging.error("HydraQueue has %s stuck items", len(past_items))
+                                #logging.error("HydraQueue has %s stuck items", len(past_items))
                                 for past_item in past_items:
-                                    logging.error("Stuck hydra_queue_id: %s", past_item['hydra_queue_id'])
-                                    #logging.error("Unclaiming stuck hydra_queue_id: %s", past_item['hydra_queue_id'])
-                                    #past_item.unclaim()
+                                    logging.error("Running stuck hydra_queue_id: %s", past_item['hydra_queue_id'])
+                                    past_item.run_now()
+                            # Any hydra_type claimed since the next_run_dttm and over 5m old are stuck. Automatically unclaim them.
+                            past_items = aegis.model.HydraType.past_items(minutes=self.stuck_minutes)
+                            if past_items and len(past_items):
+                                #logging.error("HydraType has %s stuck items", len(past_items))
+                                for past_item in past_items:
+                                    logging.error("Unclaiming stuck hydra_type_id: %s", past_item['hydra_type_name'])
+                                    past_item.unclaim()
 
                 except Exception as ex:
                     logging.exception("Batch had an inner loop failure.")
