@@ -393,12 +393,20 @@ class HydraType(aegis.database.Row):
         sql = 'UPDATE hydra_type SET claimed_dttm=NULL WHERE hydra_type_id=%s AND claimed_dttm IS NOT NULL'
         return db().execute(sql, self['hydra_type_id'])
 
-    @staticmethod
-    def clear_claims():
+    @classmethod
+    def past_items(cls, minutes=5):
         if type(db()) is aegis.database.PostgresConnection:
-            sql = "UPDATE hydra_type SET claimed_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL '5 MINUTE' AND status='live'"
+            sql = "SELECT * FROM hydra_type WHERE claimed_dttm < NOW() - INTERVAL '%s MINUTE' AND claimed_dttm > next_run_dttm" % int(minutes)
         elif type(db()) is aegis.database.MysqlConnection:
-            sql = "UPDATE hydra_type SET claimed_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL 5 MINUTE AND status='live'"
+            sql = "SELECT * FROM hydra_type WHERE claimed_dttm < NOW() - INTERVAL %s MINUTE AND claimed_dttm > next_run_dttm" % int(minutes)
+        return db().query(sql, cls=cls)
+
+    @staticmethod
+    def clear_claims(minutes=5):
+        if type(db()) is aegis.database.PostgresConnection:
+            sql = "UPDATE hydra_type SET claimed_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL '%s MINUTE' AND status='live'" % int(minutes)
+        elif type(db()) is aegis.database.MysqlConnection:
+            sql = "UPDATE hydra_type SET claimed_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL %s MINUTE AND status='live'" % int(minutes)
         return db().execute(sql)
 
 
@@ -504,19 +512,19 @@ class HydraQueue(aegis.database.Row):
         return db().execute(sql)
 
     @staticmethod
-    def clear_claims():
+    def clear_claims(minutes=15):
         if type(db()) is aegis.database.PostgresConnection:
-            sql = "UPDATE hydra_queue SET claimed_dttm=NULL, start_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL '15 MINUTE' AND finish_dttm IS NULL"
+            sql = "UPDATE hydra_queue SET claimed_dttm=NULL, start_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL '%s MINUTE' AND finish_dttm IS NULL" % int(minutes)
         elif type(db()) is aegis.database.MysqlConnection:
-            sql = "UPDATE hydra_queue SET claimed_dttm=NULL, start_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL 15 MINUTE AND finish_dttm IS NULL"
+            sql = "UPDATE hydra_queue SET claimed_dttm=NULL, start_dttm=NULL WHERE claimed_dttm < NOW() - INTERVAL %s MINUTE AND finish_dttm IS NULL" % int(minutes)
         return db().execute(sql)
 
     @classmethod
-    def past_items(cls):
+    def past_items(cls, minutes=15):
         if type(db()) is aegis.database.PostgresConnection:
-            sql = "SELECT * FROM hydra_queue WHERE work_dttm < NOW() - INTERVAL '5 MINUTE'"
+            sql = "SELECT * FROM hydra_queue WHERE work_dttm < NOW() - INTERVAL '%s MINUTE'" % int(minutes)
         elif type(db()) is aegis.database.MysqlConnection:
-            sql = "SELECT * FROM hydra_queue WHERE work_dttm < NOW() - INTERVAL 5 MINUTE"
+            sql = "SELECT * FROM hydra_queue WHERE work_dttm < NOW() - INTERVAL %s MINUTE" % int(minutes)
         return db().query(sql, cls=cls)
 
     def run_now(self):
@@ -634,7 +642,10 @@ class Build(aegis.database.Row):
 
     @classmethod
     def scan_dead_builds(cls):
-        sql = "SELECT * FROM build WHERE delete_dttm IS NOT NULL OR (deploy_dttm IS NULL AND create_dttm < NOW() - INTERVAL 1 WEEK)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql = "SELECT * FROM build WHERE delete_dttm IS NOT NULL OR (deploy_dttm IS NULL AND create_dttm < NOW() - INTERVAL '1 WEEK')"
+        elif type(db()) is aegis.database.MysqlConnection:
+            sql = "SELECT * FROM build WHERE delete_dttm IS NOT NULL OR (deploy_dttm IS NULL AND create_dttm < NOW() - INTERVAL 1 WEEK)"
         return db().query(sql, cls=cls)
 
     @classmethod
@@ -661,6 +672,8 @@ class Cache(aegis.database.Row):
             return db().execute(sql, cache_key, cache_json, cache_expiry)
         except aegis.database.PgsqlUniqueViolation as ex:
             logging.warning("Ignoring duplicate key in cache")
+        except aegis.database.MysqlIntegrityError as ex:
+            logging.warning("Ignoring duplicate key in cache")
 
     @classmethod
     def get_key(cls, cache_key):
