@@ -5,7 +5,7 @@
 
 # Python Imports
 import asyncio
-import configparser
+import base64
 import copy
 import datetime
 import functools
@@ -13,11 +13,9 @@ import json
 import logging
 import os
 import signal
-import socket
 import sys
 import time
 import traceback
-import urllib
 
 # Extern Imports
 import requests
@@ -37,7 +35,14 @@ import config
 
 
 class AegisHandler(tornado.web.RequestHandler):
+
     def __init__(self, *args, **kwargs):
+        # Initialize timer_obj on self, since this could be called before a subclass calls on self.timer_obj
+        self._parent_timer = not hasattr(self, 'timer_obj')
+        if not hasattr(self, 'timer_obj'):
+            self.timer_obj = aegis.stdlib.TimerObj()
+        if self._parent_timer:
+            aegis.stdlib.timer_start(self.timer_obj, 'init')
         super(AegisHandler, self).__init__(*args, **kwargs)
         self.tmpl = {}
         self.tmpl['logw'] = self.logw = aegis.stdlib.logw
@@ -78,8 +83,12 @@ class AegisHandler(tornado.web.RequestHandler):
         self.models = {}
         self.models['UserAgent'] = aegis.model.UserAgent
         self.models['User'] = aegis.model.User
+        if self._parent_timer:
+            aegis.stdlib.timer_stop(self.timer_obj, 'init')
 
     def prepare(self):
+        if self._parent_timer:
+            aegis.stdlib.timer_start(self.timer_obj, 'prepare')
         self.set_header('Cache-Control', 'no-cache, no-store')
         self.set_header('Pragma', 'no-cache')
         self.set_header('Expires', 'Fri, 21 Dec 2012 03:08:13 GMT')
@@ -88,15 +97,27 @@ class AegisHandler(tornado.web.RequestHandler):
         self.request.args = dict([(key, self.get_argument(key, strip=False)) for key, val in self.request.arguments.items()])
         self.setup_user()
         super(AegisHandler, self).prepare()
+        if self._parent_timer:
+            aegis.stdlib.timer_stop(self.timer_obj, 'prepare')
 
     def finish(self, chunk=None):
+        if self._parent_timer:
+            aegis.stdlib.timer_start(self.timer_obj, 'finish')
         auth_ck = self.cookie_get('auth')
         logged_out = (self.tmpl.get('logged_out') == True)
         if auth_ck and not logged_out:
             # If there's a member_auth, refresh the expiration in the backend along with the cookie
             if hasattr(self, '_member_auth'):
                 self._member_auth.refresh(options.cookie_durations['auth'] * 86400)
-            self.cookie_set('auth', auth_ck)
+            # Update auth cookie as long as it isn't being overwritten
+            new_cookie_value = self._new_cookie.get(self.cookie_name('auth'))
+            if not new_cookie_value:
+                self.cookie_set('auth', auth_ck)
+            else:
+                new_cookie_value = new_cookie_value.value.split(':')[-1].split('|')[0]
+                decoded = self.cookie_decode(base64.b64decode(new_cookie_value))
+                if decoded == auth_ck:
+                    self.cookie_set('auth', auth_ck)
         if 'session_ck' in self.tmpl:
             if self.tmpl.get('session_ck'):
                 self.cookie_set('session', self.tmpl['session_ck'])
@@ -109,8 +130,15 @@ class AegisHandler(tornado.web.RequestHandler):
                 for cookie in self._new_cookie.values():
                     cookies.append("Set-Cookie: %s" % cookie.OutputString(None))
             self.logw(cookies, "HTTP Reponse Set-Cookie Header")
+        if self._parent_timer:
+            aegis.stdlib.timer_stop(self.timer_obj, 'finish')
         super(AegisHandler, self).finish(chunk)
 
+    def on_finish(self):
+        # This runs after the response has been sent to the client, intended for cleanup.
+        #logging.warning("ON FINISH")
+        pass
+
     def debug_request(self):
         req_str = str(self.request).rstrip(')') + ', headers={'
         for header in sorted(self.request.headers.items()):
@@ -119,7 +147,7 @@ class AegisHandler(tornado.web.RequestHandler):
         logging.warning(req_str)
 
     def setup_user(self):
-        # Set up user-cookie tracking system, based on user-agent
+        # Set up user-cookie tracking system, based on user-agent. This function takes ~2-5ms depending on cpu and database speed and latency.
         if not self.tmpl['user_agent']:
             self.tmpl['user_agent'] = 'NULL USER AGENT'
         self.tmpl['user_agent_obj'] = ua = user_agents.parse(self.tmpl['user_agent'])
@@ -183,12 +211,18 @@ class AegisHandler(tornado.web.RequestHandler):
         return options.template_path
 
     def render(self, template_name, **kwargs):
+        aegis.stdlib.timer_start(self.timer_obj, 'render')
         template_path = os.path.join(options.template_path, template_name)
-        return super(AegisHandler, self).render(template_path, **kwargs)
+        # Override parent class render to remove the embeds and instrument a timer here. Copied in from tornado/web.py render()
+        if self._finished:
+            raise RuntimeError("Cannot render() after finish()")
+        html = self.render_string(template_name, **kwargs)
+        aegis.stdlib.timer_stop(self.timer_obj, 'render')
+        self.finish(html)
 
     def render_path(self, template_name, **kwargs):
         template_path = os.path.join(self.get_template_path(), template_name)
-        return super(AegisHandler, self).render(template_path, **kwargs)
+        self.render(template_path, **kwargs)
 
     def _handle_request_exception(self, ex):
         aegis.model.db().close()  # Closing database effectively does a transaction ROLLBACK
@@ -554,34 +588,48 @@ class AegisApplication():
             log_method = tornado.log.access_log.warning
         else:
             log_method = tornado.log.access_log.error
-        request_time = 1000.0 * handler.request.request_time()
+        # Main part of the response
         host = handler.request.host.split(':')[0]
         extra_debug = ''
         user_id = None
-        member_id = None
         if hasattr(handler, 'tmpl'):
             user_id = handler.tmpl.get('user', {}).get('user_id')
-            member_id = None
-            if handler.tmpl.get('member'):
-                member_id = handler.tmpl['member'].get('member_id')
-            extra_debug = '| uid: %s | mid: %s' % (user_id or '-', member_id or '-')
+            extra_debug = '| uid: %s | mid: %s' % (user_id or '-', handler.get_member_id() or '-')
             if hasattr(handler, 'json_length'):
                 extra_debug += ' | kb: %4.2f' % (handler.json_length / 1024.0)
             extra_debug = aegis.stdlib.cstr(extra_debug, 'yellow')
             if handler.user_is_robot():
                 extra_debug += aegis.stdlib.cstr('   BOT', 'blue')
-        log_method("%s %d %s %.2fms %s", host, handler.get_status(), handler._request_summary(), request_time, extra_debug)
+        # Timing
+        request_t_ms = handler.request.request_time() * 1000   # Use tornado's from start of __init__ to end of finish() as a reference
+        timers = None
+        if hasattr(handler, 'timer_obj'):
+            timers = handler.timer_obj._timers
+        log_method("%s %d %s %.2fms %s", host, handler.get_status(), handler._request_summary(), request_t_ms, extra_debug)
+        # If request takes over options.slow_req_ms we can give some extra debug output
+        slow_req_ms = aegis.config.get('slow_req_ms') or 250
+        if request_t_ms > slow_req_ms and timers:
+            net_t_ms = timers.get('_network_exec_s', 0) * 1000
+            db_t_ms = timers.get('_database_exec_s', 0) * 1000
+            cpu_t_ms = request_t_ms - net_t_ms - db_t_ms
+            init_t_ms = timers.get('_init_exec_s', 0) * 1000
+            prepare_t_ms = timers.get('_prepare_exec_s', 0) * 1000
+            render_t_ms = timers.get('_render_exec_s', 0) * 1000
+            finish_t_ms = timers.get('_finish_exec_s', 0) * 1000
+            handler_t_ms = request_t_ms - init_t_ms - prepare_t_ms - render_t_ms - finish_t_ms
+            msg = "Req Time: %.3fms  |  %.3fms cpu  %.3fms db  %.3fms net  |  %.3fms init  %.3fms prepare  %.3fms handler   %.3fms render  %.3fms finish"
+            msg = msg % (request_t_ms, cpu_t_ms, db_t_ms, net_t_ms, init_t_ms, prepare_t_ms, handler_t_ms, render_t_ms, finish_t_ms)
+            tornado.log.access_log.warning(msg)
+
 
 
 def sig_handler(sig, frame):
     io_loop = tornado.ioloop.IOLoop.instance()
-
     def stop_loop():
         if len(asyncio.Task.all_tasks(io_loop)) == 0:
             io_loop.stop()
         else:
             io_loop.call_later(1, stop_loop)
-
     io_loop.add_callback_from_signal(stop_loop)
 
 
@@ -888,7 +936,7 @@ class AegisBuild(AegisWeb):
             if build_id:
                 build = aegis.model.Build.get_id(build_id)
                 build.set_soft_deleted()
-                return self.redirect('/aegis/build')
+                return self.redirect('/admin/build')
 
 
 class AegisBuildForm(AegisWeb):
